/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-16 17:50:31.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.controller.version_1_0;

import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.model.UserInfo;
import cn.com.qmth.examcloud.app.service.UserAuthService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

import static cn.com.qmth.examcloud.app.model.Constants.COMMON_ACCOUNT_TYPE;

/**
 * 认证中心相关接口
 *
 * @Version v1.0
 */
@RestController
@RequestMapping("/api/v1")
@Api(tags = "认证中心相关接口")
public class UserAuthRestController {
    private final static Logger log = LoggerFactory.getLogger(UserAuthRestController.class);
    @Autowired
    private UserAuthService userAuthService;

    @ApiOperation(value = "登录接口")
    @RequestMapping(value = "/user/login", method = {RequestMethod.GET, RequestMethod.POST})
    public Result<UserInfo> login(@RequestParam String account, @RequestParam String password, @RequestParam(required = false) String accountType,
                                  @RequestParam(required = false) Long rootOrgId, @RequestParam(required = false) String domain, HttpServletRequest request) throws Exception {
        if (accountType == null) {
            accountType = COMMON_ACCOUNT_TYPE;
        }
        Result<UserInfo> result = userAuthService.login(account, password, accountType, rootOrgId, domain);
        /*if (result.isSuccess()) {
            //获取当前登录用户信息
            UserInfo userInfo = result.getData();
            UserToken userToken = new UserToken(account, password, accountType, rootOrgId, domain, userInfo.getKey(), userInfo.getToken());

            //存放登录信息到Session中
            String sessionKey = StrUtils.uuid();
            HttpSession session = request.getSession();
            //session.setMaxInactiveInterval(EXPIRE_TIME);
            session.setMaxInactiveInterval(60);
            session.setAttribute(sessionKey, userToken);
            log.debug("token:" + userToken.getToken() + " sessionId:" + sessionKey);

            //替换Token为本地SessionKey
            userInfo.setToken(sessionKey);
        }*/
        return result;
    }

    @ApiOperation(value = "登出接口")
    @RequestMapping(value = "/user/logout", method = {RequestMethod.GET, RequestMethod.POST})
    public Result logout(@RequestHeader String key, @RequestHeader String token) throws Exception {
        return userAuthService.logout(key, token);
    }

    @ApiOperation(value = "获取用户信息接口")
    @RequestMapping(value = "/user/info", method = {RequestMethod.GET, RequestMethod.POST})
    public Result getUserInfo(@RequestHeader String key, @RequestHeader String token) throws Exception {
        return userAuthService.getUserInfo(key, token);
    }

    @ApiOperation(value = "修改密码接口")
    @RequestMapping(value = "/user/update/password", method = {RequestMethod.GET, RequestMethod.POST})
    public Result updatePassword(@RequestHeader String key, @RequestHeader String token, @RequestParam Long userId, @RequestParam String password) throws Exception {
        return userAuthService.updatePassword(key, token, userId, password);
    }

    @ApiOperation(value = "手机号登录接口", hidden = true)
    @RequestMapping(value = "/user/login/by/phone", method = {RequestMethod.GET, RequestMethod.POST})
    public Result loginByPhone(@RequestParam String phone, @RequestParam String password) throws Exception {
        return userAuthService.loginByPhone(phone, password);
    }

    @ApiOperation(value = "保存用户绑定的手机号接口", hidden = true)
    @RequestMapping(value = "/user/binding/phone", method = {RequestMethod.GET, RequestMethod.POST})
    public Result userBindingPhone(@RequestHeader String key, @RequestHeader String token, @RequestParam Long userId, @RequestParam String phone, @RequestParam String code) throws Exception {
        return userAuthService.userBindingPhone(key, token, userId, phone, code);
    }

}