/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-20 14:15:02.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.core.config;

import cn.com.qmth.examcloud.app.model.DeviceRecord;
import cn.com.qmth.examcloud.app.service.DeviceRecordService;
import cn.com.qmth.examcloud.app.service.UserAuthService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.Map;

import static cn.com.qmth.examcloud.app.model.Constants.PASSWORD;

public class AccessInterceptor extends HandlerInterceptorAdapter {
    private final static Logger log = LoggerFactory.getLogger(AccessInterceptor.class);
    @Autowired
    private DeviceRecordService deviceRecordService;
    @Autowired
    private UserAuthService userAuthService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        DeviceRecord record = this.parseParams(request);
        /*if (StringUtils.isNotEmpty(record.getLoginToken())) {
            //带Token时则需要认证
            HttpSession session = request.getSession();
            System.out.println(record.getLoginToken() + " " + session.getMaxInactiveInterval());

            //获取原来的登录信息，并判断是否再有效时间内
            UserToken userToken = (UserToken) session.getAttribute(record.getLoginToken());
            if (userToken != null && !userToken.hasExpired()) {
                //登录已失效，则Token续期
                Result<UserInfo> result = userAuthService.login(userToken.getAccount(), userToken.getPassword(),
                        userToken.getAccountType(), userToken.getRootOrgId(), userToken.getDomain());
                UserInfo userInfo = result.getData();
                if (userInfo != null) {
                    log.debug("new token is " + userInfo.getToken());
                    userToken.setToken(userInfo.getToken());
                    session.setAttribute(record.getLoginToken(), userToken);
                }
            }
        }*/
        //异步保存设备访问记录
        deviceRecordService.addDeviceRecord(record);
        return true;
    }

    private DeviceRecord parseParams(HttpServletRequest request) {
        String system = request.getHeader("system");
        String deviceId = request.getHeader("deviceId");
        String netType = request.getHeader("netType");
        String brand = request.getHeader("brand");
        String model = request.getHeader("model");
        String sysVersion = request.getHeader("sysVersion");
        String appVersion = request.getHeader("appVersion");
        String appCode = request.getHeader("appCode");
        String patchCode = request.getHeader("patchCode");
        String account = request.getHeader("account");
        String key = request.getHeader("key");
        String token = request.getHeader("token");
        String params = this.getParams(request);
        String url = request.getServletPath() + params;
        String ip = this.getIp(request);

        DeviceRecord record = new DeviceRecord();
        record.setSystem(system);
        record.setDeviceId(deviceId);
        record.setNetType(netType);
        record.setBrand(brand);
        record.setModel(model);
        record.setSysVersion(sysVersion);
        record.setAppVersion(appVersion);
        record.setAppCode(appCode);
        record.setPatchCode(patchCode);
        record.setAccount(account);
        record.setLoginKey(key);
        record.setLoginToken(token);
        record.setUrl(url);
        record.setIp(ip);
        record.setCreateDate(new Date());
        return record;
    }

    private String getParams(HttpServletRequest request) {
        StringBuilder params = new StringBuilder();
        Map<String, String[]> map = request.getParameterMap();
        for (Map.Entry<String, String[]> entry : map.entrySet()) {
            String name = entry.getKey();
            String[] values = entry.getValue();
            params.append("&").append(name).append("=");
            if (values.length > 0) {
                if (PASSWORD.equalsIgnoreCase(name)) {
                    params.append("***");
                } else {
                    params.append(values[0]);
                }
            }
        }
        /*try {
            //将RequestBody中的参数转为JSON格式(注意：request.getInputStream()执行一次后则导致Controller再无法取到RequestBody值)
            byte[] requestBody = StreamUtils.copyToByteArray(request.getInputStream());
            JSONObject object = JSON.parseObject((new String(requestBody, "UTF-8")));
            if (object != null) {
                System.out.println(object.toJSONString());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }*/
        return params.toString().replaceFirst("&", "?");
    }

    private String getIp(HttpServletRequest request) {
        String ip = request.getHeader("X-Forwarded-For");
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_CLIENT_IP");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_X_FORWARDED_FOR");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }

}